<?php

namespace App\Service;

use App\Model\Depart;
use App\Model\Employee;
use App\Model\Product;
use App\Model\PurchaseOrder;
use App\Model\PurchaseOrderInfo;
use App\Model\SalesOrder;
use Illuminate\Support\Facades\DB;

/**
 * 采购单
 */
class PurchaseOrderService extends Service
{
    /**
     * 采购单编辑
     * @param $data
     * @param $user
     * @return array
     */
    public function edit($data,$user){
        list($status,$msg) = $this->orderRule($data,false);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = PurchaseOrder::where('order_number',$data['order_number'])->first();
            $material_model->order_number = $data['order_number'];
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->rate = $data['rate'] ?? 0;
            $material_model->total = $data['total'] ?? 0;
            $material_model->other_fee = $data['other_fee'] ?? 0;
            $material_model->discount_fee = $data['discount_fee'] ?? 0;
            $material_model->purchase_total = $data['purchase_total'] ?? 0;
            $material_model->supplier = $data['supplier'] ?? '';
            $material_model->purchase_id = $data['purchase_id'] ?? 0;
            $material_model->order_type = $data['order_type'];
            $material_model->sales_order_id = $data['sales_order_id'] ?? 0;
            $material_model->mark = $data['mark'] ?? '';
            $material_model->save();

            PurchaseOrderInfo::where('order_number',$data['order_number'])
                ->where('del_time',0)
                ->update(['del_time' => time()]);
            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'purchase_order_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'depart_id' => $data['depart_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'price' => $value['price'],
                        'mark' => $value['mark'] ?? '',
                        'title' => $value['title'] ?? '',
                        'code' => $value['code'] ?? '',
                        'size' => $value['size'] ?? '',
                        'unit' => $value['unit'] ?? 0,
                        'bar_code' => $value['bar_code'] ?? '',
                    ];
                }
                PurchaseOrderInfo::insert($sub);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 采购单新增
     * @param $data
     * @param $user
     * @return array
     */
    public function add($data,$user){
        list($status,$msg) = $this->orderRule($data);
        if(!$status) return [$status,$msg];
        try{
            DB::beginTransaction();
            $material_model = new PurchaseOrder();
            $material_model->order_number = $data['order_number'];
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->rate = $data['rate'] ?? 0;
            $material_model->total = $data['total'] ?? 0;
            $material_model->other_fee = $data['other_fee'] ?? 0;
            $material_model->discount_fee = $data['discount_fee'] ?? 0;
            $material_model->purchase_total = $data['purchase_total'] ?? 0;
            $material_model->supplier = $data['supplier'] ?? '';
            $material_model->purchase_id = $data['purchase_id'] ?? 0;
            $material_model->order_type = $data['order_type'];
            $material_model->sales_order_id = $data['sales_order_id'] ?? 0;
            $material_model->mark = $data['mark'] ?? '';
            $material_model->crt_id = $user['id'];
            $material_model->save();

            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'purchase_order_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'depart_id' => $data['depart_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'price' => $value['price'],
                        'mark' => $value['mark'] ?? '',
                        'title' => $value['title'] ?? '',
                        'code' => $value['code'] ?? '',
                        'size' => $value['size'] ?? '',
                        'unit' => $value['unit'] ?? 0,
                        'bar_code' => $value['bar_code'] ?? '',
                    ];
                }
                PurchaseOrderInfo::insert($sub);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 采购单详情
     * @param $data
     * @return array
     */
    public function detail($data){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据'];

        $order = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->select('id','order_number','depart_id','mark','state','crt_id','order_type','sales_order_id','supplier','purchase_id','rate','total','other_fee','discount_fee','purchase_total','crt_time')
            ->first();
        if(empty($order)) return [false, '采购订单不存在或已被删除'];
        $order = $order->toArray();
        $order['order_type_title'] = PurchaseOrder::$order_type[$order['order_type']] ?? '';
        $order['order_type_title'] = PurchaseOrder::$order_type[$order['order_type']] ?? '';
        $order['state_title'] = PurchaseOrder::$name[$order['state']] ?? '';
        $sales = SalesOrder::where('id',$order['sales_order_id'])->value('order_number');
        $order['sales_order_number'] = $sales;
        $emp_map = Employee::whereIn('id',array_unique(array_merge_recursive([$order['purchase_id']],[$order['crt_id']])))
            ->pluck('emp_name','id')
            ->toArray();
        $order['purchase_title'] = $emp_map[$order['purchase_id']] ?? '';

        $info = PurchaseOrderInfo::where('del_time',0)
            ->where('order_number',$data['order_number'])
            ->select('id','order_number','depart_id','product_id','number','mark','price','title','size','code','bar_code','purchase_order_id')
            ->get()->toArray();
        $order['product'] = $info;
        $order['crt_name'] = $emp_map[$order['crt_id']] ?? '';
        $order['crt_time'] = $order['crt_time'] ? date("Y-m-d H:i:s",$order['crt_time']): '';

        return [true, $order];
    }

    /**
     * 采购单删除
     * @param $data
     * @return array
     */
    public function del($data){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据！'];
        $order = PurchaseOrder::where('del_time',0)->where('order_number',$data['order_number'])->first();
        if(empty($order)) return [false,'采购单不存在或已被删除'];
        $order = $order->toArray();
        if($order['state'] > PurchaseOrder::STATE_ZERO) return [false,'请确认采购订单状态,删除失败！'];

        try{
            DB::beginTransaction();
            PurchaseOrder::where('order_number',$data['order_number'])->update([
                'del_time'=>time()
            ]);
            PurchaseOrderInfo::where('order_number',$data['order_number'])->update([
                'del_time'=>time()
            ]);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 获取采购单列表
     * @param $data
     * @param $user
     * @return array
     */
    public function getList($data,$user){
        $model = PurchaseOrder::where('del_time',0)
            ->select('order_number','id','depart_id','mark','crt_time','state','crt_id','order_type','sales_order_id','supplier','purchase_id','purchase_total','rate','discount_fee','other_fee','total')
            ->orderby('id', 'desc');
        if(! empty($data['order_number'])) $model->where('order_number', 'LIKE', '%'.$data['order_number'].'%');
        if(! empty($data['depart_id'])) $model->where('depart_id', $data['depart_id']);
        if(! empty($data['company_id'])) $model->where('company_id', $data['company_id']);
        if(isset($data['state'])) $model->where('state',$data['state']);
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->whereBetween('crt_time',[$return[0],$return[1]]);
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillListData($list);

        return [true, $list];
    }

    /**
     * 拼接数据
     * @param $data
     * @return array
     */
    public function fillListData($data){
        if(empty($data['data'])) return $data;

        $depart_id = array_unique(array_column($data['data'],'depart_id'));
        $depart_map = Depart::whereIn('id',$depart_id)
            ->pluck('title','id')
            ->toArray();

        $emp_id = array_unique(array_merge_recursive(array_column($data['data'],'crt_id'),array_column($data['data'],'purchase_id')));
        $emp_map = Employee::whereIn('id',$emp_id)
                    ->pluck('emp_name','id')
                    ->toArray();

        $sales = SalesOrder::whereIn('id',array_unique(array_column($data['data'],'sales_order_id')))->pluck('order_number','id')->toArray();
        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date("Y-m-d H:i:s",$value['crt_time']) : '';
            $data['data'][$key]['depart_name'] = $depart_map[$value['depart_id']] ?? '';
            $data['data'][$key]['crt_name'] = $emp_map[$value['crt_id']] ?? '';
            $data['data'][$key]['purchase_name'] = $emp_map[$value['purchase_id']] ?? '';
            $data['data'][$key]['order_type_title'] = PurchaseOrder::$order_type[$value['order_type']] ?? '';
            $data['data'][$key]['state_title'] = PurchaseOrder::$name[$value['state']] ?? '';
            $data['data'][$key]['sales_order_number'] = $sales[$value['sales_order_id']] ?? '';
        }

        return $data;
    }

    /**
     * 采购单参数规则
     * @param $data
     * @param $is_check
     * @return array
     */
    public function orderRule(&$data, $is_check = true){
        if(empty($data['order_type'])) return [false,'采购订单类型不能为空'];
        if($this->isEmpty($data,'product')) return [false,'产品不能为空'];
        if(! empty($data['rate'])){
            $res = $this->checkNumber($data['rate']);
            if(! $res) return [false, '整单扣除率请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['other_fee'])){
            $res = $this->checkNumber($data['other_fee']);
            if(! $res) return [false, '其它费用请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['discount_fee'])){
            $res = $this->checkNumber($data['discount_fee']);
            if(! $res) return [false, '优惠金额请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['total'])){
            $res = $this->checkNumber($data['total']);
            if(! $res) return [false, '产品总价请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['purchase_total'])){
            $res = $this->checkNumber($data['purchase_total']);
            if(! $res) return [false, '采购总价请输入不超过两位小数并且大于0的数值'];
        }
        foreach ($data['product'] as $value){
            if(empty($value['number'])) return [false,'产品数量不能为空'];
            $res = $this->checkNumber($value['number']);
            if(! $res) return [false,'请输入正确的产品数量'];
        }

        if($is_check){
            $order_number = (new OrderNoService())->createOrderNumber(PurchaseOrder::prefix);
            if(empty($order_number)) return [false,'采购单号生成失败！'];
            $data['order_number'] = $order_number;
        }else{
            if($this->isEmpty($data,'order_number')) return [false,'采购单号不能为空！'];
        }

        return [true, ''];
    }

    /**
     * 采购单确认
     * @param $data
     * @param $user
     * @return array
     */
    public function purchaseOrderConfirm($data,$user){
        if(empty($data['order_number'])) return [false, '请选择数据'];

        $order = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->first();
        if(empty($order)) return [false, '采购订单不存在或已被删除'];
        if($order->state != PurchaseOrder::STATE_ZERO) return [false, '请确认采购订单状态，操作失败'];

        PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->update(['state' => PurchaseOrder::STATE_ONE]);

        return [true,''];
    }
}
